//+------------------------------------------------------------------+
//|                                          VininI_BB_MA_WPR.v1.mq4 |
//|                                                  Victor Nicolaev |
//|                                                    vinin@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright 2008. Victor Nicolaev"
#property link      "vinin@mail.ru"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  clrAqua
#property indicator_color2  clrGreen
#property indicator_color3  clrYellow
#property indicator_color4  clrYellow
#property indicator_level1  0
#property indicator_level2  60
#property indicator_level3  -60
#property indicator_maximum 100
#property indicator_minimum -100
//---- input parameters

enum enTimeFrames
{
   tf_cu  = PERIOD_CURRENT, // Current time frame
   tf_m1  = PERIOD_M1,      // 1 minute
   tf_m5  = PERIOD_M5,      // 5 minutes
   tf_m15 = PERIOD_M15,     // 15 minutes
   tf_m30 = PERIOD_M30,     // 30 minutes
   tf_h1  = PERIOD_H1,      // 1 hour
   tf_h4  = PERIOD_H4,      // 4 hours
   tf_d1  = PERIOD_D1,      // Daily
   tf_w1  = PERIOD_W1,      // Weekly
   tf_mn1 = PERIOD_MN1,     // Monthly
   tf_n1  = -1,             // First higher time frame
   tf_n2  = -2,             // Second higher time frame
   tf_n3  = -3              // Third higher time frame
};

extern enTimeFrames     TimeFrame        = tf_cu;      // Time frame
extern int              WPR_Period       = 55;
extern int              MA_Period        = 3;
extern ENUM_MA_METHOD   MA_Mode          = MODE_SMA;
extern int              BB_Period        = 89;
extern double           BB_Div           = 1.0;
input bool              arrowsVisible    = false;                // Arrows visible true/false?
input bool              arrowsOnNewest   = false;                // Arrows drawn on newest bar of higher time frame bar true/false?
input string            arrowsIdentifier = "wpr Arrows1";        // Unique ID for arrows
input double            arrowsUpperGap   = 0.5;                  // Upper arrow gap
input double            arrowsLowerGap   = 0.5;                  // Lower arrow gap
input color             arrowsUpColor    = clrBlue;              // Up arrow color
input color             arrowsDnColor    = clrCrimson;           // Down arrow color
input int               arrowsUpCode     = 116;                  // Up arrow code
input int               arrowsDnCode     = 116;                  // Down arrow code
input int               arrowsUpSize     = 2;                    // Up arrow size
input int               arrowsDnSize     = 2;                    // Down arrow size
input bool              Interpolate      = true;                 // Interpolated in mtf mode


double WPR[];
double WPRMidle[];
double WPRUP[];
double WPRDN[];
double MA[],valt[],count[];
string indicatorFileName;
#define _mtfCall(_buff,_ind) iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,WPR_Period,MA_Period,MA_Mode,BB_Period,BB_Div,arrowsVisible,arrowsOnNewest,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsUpSize,arrowsDnSize,_buff,_ind)
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init() 
{
   IndicatorBuffers(7);
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexBuffer(0,MA);
   SetIndexBuffer(1,WPRMidle);
   SetIndexBuffer(2,WPRUP);
   SetIndexBuffer(3,WPRDN);
   SetIndexBuffer(4,WPR);
   SetIndexBuffer(5,valt);
   SetIndexBuffer(6,count);
   
   indicatorFileName = WindowExpertName();
   TimeFrame         = (enTimeFrames)timeFrameValue(TimeFrame);
   IndicatorShortName(timeFrameToString(TimeFrame)+" MA_WPR("+WPR_Period+","+MA_Period+")");
   SetIndexLabel(0,"MA(WPR)");

return(0); 
}
void OnDeinit(const int reason)
{ 
    string lookFor       = arrowsIdentifier+":";
    int    lookForLength = StringLen(lookFor);
    for (int i=ObjectsTotal()-1; i>=0; i--)
    {
       string objectName = ObjectName(i);
       if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
    }
}

//+------------------------------------------------------------------+
int start() 
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = fmin(Bars-counted_bars,Bars-1); count[0] = limit;   
         if (TimeFrame != Period())
         {
           limit = (int)MathMax(limit,MathMin(Bars-1,_mtfCall(6,0)*TimeFrame/_Period));
           for(i=limit; i>=0 && !_StopFlag; i--)
           {
              int y = iBarShift(NULL,TimeFrame,Time[i]);
                 MA[i]       = _mtfCall(0,y);
                 WPRMidle[i] = _mtfCall(1,y);
                 WPRUP[i]    = _mtfCall(2,y);
                 WPRDN[i]    = _mtfCall(3,y);
              
                 //
                 //
                 //
                     
                 if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;
                   #define _interpolate(buff) buff[i+k] = buff[i]+(buff[i+n]-buff[i])*k/n
                   int n,k; datetime btime = iTime(NULL,TimeFrame,y);
                      for(n = 1; (i+n)<Bars && Time[i+n] >= btime; n++) continue;	
                      for(k = 1; k<n && (i+n)<Bars && (i+k)<Bars; k++) 
                      {
                         _interpolate(MA); 
                         _interpolate(WPRMidle);
                         _interpolate(WPRUP);
                         _interpolate(WPRDN);
                      }                 
         }
   return(0);
   }  
   
   //
   //
   //
   
   for(i=limit;i>=0;i--) WPR[i] = (iWPR(Symbol(),Period(),WPR_Period,i)+50.0)*2.0;
   for(i=limit;i>=0;i--) MA[i] = iMAOnArray(WPR,0,MA_Period,0,MA_Mode,i);
   for(i=limit;i>=0;i--) 
   {
      double tmp= (iBandsOnArray(WPR,0,BB_Period,1,0,MODE_UPPER,i)-
                   iBandsOnArray(WPR,0,BB_Period,1,0,MODE_LOWER,i))/2.0;
      WPRMidle[i] = iMAOnArray(WPR,0,BB_Period,0,MODE_SMA,i);
      WPRUP[i]    = WPRMidle[i]+BB_Div*tmp;
      WPRDN[i]    = WPRMidle[i]-BB_Div*tmp;
      valt[i]     = (MA[i]>WPRUP[i]) ? 1 : (MA[i]<WPRDN[i]) ? -1 : 0; 
      
      //
      //
      //
      
      if (arrowsVisible)
      {
         string lookFor = arrowsIdentifier+":"+(string)Time[i]; ObjectDelete(lookFor);            
         if (i<(Bars-1) && valt[i] != valt[i+1])
         {
            if (valt[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,arrowsUpSize,false);
            if (valt[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode,arrowsDnSize, true);
         }
      }    
   }
   return(0); 
}// int start()

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------

void drawArrow(int i,color theColor,int theCode, int theSize, bool up)
{
   string name = arrowsIdentifier+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //

      datetime atime = Time[i]; if (arrowsOnNewest) atime += _Period*60-1;      
      ObjectCreate(name,OBJ_ARROW,0,atime,0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_WIDTH,theSize);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
int timeFrameValue(int _tf)
{
   int add  = (_tf>=0) ? 0 : fabs(_tf);
   if (add != 0) _tf = _Period;
   int size = ArraySize(iTfTable); 
      int i =0; for (;i<size; i++) if (iTfTable[i]==_tf) break;
                                   if (i==size) return(_Period);
                                                return(iTfTable[(int)fmin(i+add,size-1)]);
}






