//+-------------------------------------------------------------------+
//|                                                    Bandswidth.mq4 |
//|                                         by Linuxser for Forex TSD |
//|                                                                   |
//| John Bollinger original formula is:                               |
//| (Upper BB - Lower BB)/middle BB                                   |
//+-------------------------------------------------------------------+
//------------------------------------------------------------------
#property copyright "www.forex-station.com"
#property link      "www.forex-station.com"
//------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1  clrRoyalBlue


//---- input parameters
input int             BBPeriod         = 20;
input int             StdDeviation     = 2;
input bool            alertsOn         = true;                // Alerts?
input bool            alertsOnCurrent  = false;               // Alerts open bar?
input bool            alertsMessage    = true;                // Alerts message?
input bool            alertsSound      = false;               // Alerts sound?
input bool            alertsNotify     = false;               // Alerts notification?
input bool            alertsEmail      = false;               // Alerts email?
input string          soundFile        = "alert2.wav";        // Alerts sound file
//---- buffers
double val[],valc[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers(2);
   SetIndexBuffer(0,val); SetIndexStyle(0,DRAW_LINE);  SetIndexLabel(0,"Bandswidth("+BBPeriod+","+StdDeviation+")");
   SetIndexBuffer(1,valc);
   
   IndicatorShortName("Bandswidth("+BBPeriod+","+StdDeviation+")");
return(0);
}
//+------------------------------------------------------------------+
//| Momentum                                                         |
//+------------------------------------------------------------------+
int start()
{
  int i,counted_bars = IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            int limit=MathMin(Bars-counted_bars,Bars-1);
            
   for (i=limit;i>=0;i--)
   { 
      double ub = iBands(NULL,0,BBPeriod,StdDeviation,0,PRICE_CLOSE,MODE_UPPER,i); 
      double lb = iBands(NULL,0,BBPeriod,StdDeviation,0,PRICE_CLOSE,MODE_LOWER,i);
      double cl = iMA(NULL,0,BBPeriod,0,MODE_SMA,PRICE_CLOSE,i);
         val[i]  = (ub!=lb) ? (ub-lb)/cl : 0;
         valc[i] = (i<Bars-1) ? (val[i]>val[i+1]) ? 1 : (val[i]<val[i+1]) ? -1 : valc[i+1] : 0;  
   }
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (valc[whichBar] != valc[whichBar+1])
      {
         if (valc[whichBar] == 1)  doAlert(whichBar," sloping up");
         if (valc[whichBar] ==-1)  doAlert(whichBar," sloping down");
      }         
   }
return(0);
}

//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //

          message = timeFrameToString(_Period)+" "+_Symbol+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" Bollinger Bandwidth "+doWhat;
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(_Symbol+" Bollinger Bandwidth ",message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
int timeFrameValue(int _tf)
{
   int add  = (_tf>=0) ? 0 : fabs(_tf);
   if (add != 0) _tf = _Period;
   int size = ArraySize(iTfTable); 
      int i =0; for (;i<size; i++) if (iTfTable[i]==_tf) break;
                                   if (i==size) return(_Period);
                                                return(iTfTable[(int)fmin(i+add,size-1)]);
}




       